package net.ipksindia.config

import com.typesafe.config.ConfigFactory
import org.slf4j.LoggerFactory
import kotlin.system.exitProcess

object AppConfig {

    private val logger = LoggerFactory.getLogger(AppConfig::class.java)

    private val environment = ConfigFactory.load().getString("ktor.environment")
    private val config = when (environment) {
        "prod" -> ConfigFactory.load("application-prod.conf")
        else -> ConfigFactory.load("application-dev.conf")
    }

    fun validateConfig() {
        val requiredKeys = listOf(
            "database.host",
            "database.port",
            "database.name",
            "database.user",
            "database.password",
            "bank.server.protocol",
            "bank.server.host",
            "bank.server.port",
            "bank.server.rootRoute",
            "bank.server.transactionRoute",
            "bank.codes"
        )

        val missingKeys = requiredKeys.filterNot { config.hasPath(it) }

        if (missingKeys.isNotEmpty()) {
            logger.error("Missing configuration keys: {}", missingKeys.toString())
            exitProcess(1)
        }

        val dbPort = config.getInt("database.port")
        val remoteServerPort = config.getInt("bank.server.port")
        if (dbPort !in 1..65535) {
            logger.error("Invalid database port: {}", dbPort)
            exitProcess(1)
        }

        if (remoteServerPort !in 1..65535) {
            logger.error("Invalid remote server port: {}", remoteServerPort)
            exitProcess(1)
        }
    }

    val databaseConfig by lazy { DatabaseConfig(
        config.getString("database.host"),
        config.getInt("database.port"),
        config.getString("database.name"),
        config.getString("database.user"),
        config.getString("database.password")
    )}

    val remoteServerConfig by lazy {
        RemoteServerConfig(
            config.getString("bank.server.protocol"),
            config.getString("bank.server.host"),
            config.getInt("bank.server.port"),
            config.getString("bank.server.rootRoute"),
            config.getString("bank.server.transactionRoute")
        )
    }

    val bankCodes: MutableList<String> by lazy { config.getStringList("bank.codes")}
}

data class RemoteServerConfig(
    val protocol: String,
    val host: String,
    val port: Int,
    val rootRoute: String,
    val transactionRoute: String
) {
    val transactionUrl = "$protocol://$host:$port/$rootRoute/$transactionRoute"
}

data class DatabaseConfig(
    val host: String,
    val port: Int,
    val name: String,
    val user: String,
    val password: String
) {
    val url = "jdbc:oracle:thin:@$host:$port:$name"
}